import 'dart:convert';
import 'dart:io' show Platform;
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

final channel = MethodChannel('itzam_socket_io');

typedef void OnWS(String eventName, dynamic data);

class SocketIO {
  OnWS onWS = null;

  void connect({@required String host, Map<String, dynamic> query}) async {
    var queryParams = "?";
    var dynamicQuery = {};
    var index = 0;
    query.forEach((key, value) {
      queryParams +=
          "$key=${Uri.encodeFull(value.toString())}${index + 1 < query.length ? '&' : ''}";
      dynamicQuery[key] = value;
      index++;
    });

    await channel.invokeMethod("connect",
        {'host': host, 'query': dynamicQuery, "params": queryParams});

    // ignore: missing_return
    channel.setMethodCallHandler((call) {
      if (call.method == 'incoming') {
        final String eventName = call.arguments['eventName'];
        final data = call.arguments['data'];

        if (Platform.isIOS) {
          onWS(eventName, data);
        } else {
          try {
            final jsonObject = jsonDecode(data);
            onWS(eventName, jsonObject);
          } catch (e) {
            onWS(eventName, data);
          }
        }
      }
    });
  }

  void disconnect() async {
    await channel.invokeMethod("disconnect");
  }

  on(String eventName) async {
    await channel.invokeMethod("on", {"eventName": eventName});
  }

  emit({@required String eventName, @required dynamic data}) async {
    await channel.invokeMethod("emit", {"eventName": eventName, "data": data});
  }
}
